{% extends base %}
{% block control %}
<textarea data-piecewise="blur" id="{{ id }}" name="{{ name }}" class="{{ css_class }}" style="{{ css_style }}" rows="{{ rows }}" columns="{{ columns }}"  placeholder="{{ placeholder }}"></textarea>
{% endblock %}
