{% extends base %}
{% block control %}
<select data-piecewise="blur" id="{{ id }}" name="{{ name }}">
    {% for item in items %}
    <option value="{{item[0]}}">{{item[1]}}</option>
    {% endfor %}
</select>
{% endblock %}
