<div class="control-group">
    {% block error %}
        {% if errors %}
        <div class="alert alert-error">
            {{ errors[0]['message'] }}
        </div>
        {% endif %}
        {% if g.ajax %}
        <div class="alert alert-error" style="display:none;" id="{{ id }}_error">
        </div>
        {% endif %}
    {% endblock %}
    {% if label %}
        {% block label %}
            <label class="control-label" for="{{ name }}">{{ title }}</label>
        {% endblock %}
    {% endif %}
    <div class="controls">
        {% block control %}
        {% endblock %}
    </div>
</div>
